function alfa_i =  h_matrix(A,xL,xU)
% H_MATRIX finds a constrained minimum of a function of several variables.
%
% The H-matrix method can be used for the identification of an appropriate
% diagonal shift matrix delta for the interval matrix [A].
%
% An iterative approach can be used to compute matrix delta which satisfies
% the above conditions.
%
% ALFA_I = H_MATRIX(A,xL,xU) will find a different alfa value. Function 
% accepts input A, xL a nd xU, where [A] is a symmetric interval matrix,xL
% is a lower bound and xU is an upper bound of objective function.
% xL and xU could be a scalar or vector.
%
% Created by Andrea Szakalova on 6.11.2011

% xL = [-1;-1];
% xU = [2;1];

n = length(A);
% Step 1: Use E-matrix method to find alfa_E; construct delta_E;
%         calculate d_E_max; set k=1 (original k=0)
alfa_E = max(0,-1/2*e_matrix(A,1));
delta_E = alfa_E*eye(n);
d_E_max = 1/4*alfa_E*sum((xU - xL).^2);
k = 1;

% Step 2: Compute a modified Cholesky decomposition of A_midpoint_modif:
% ----- modified midpoint matrix ----- 
A_dol = A.inf;
A_midpoint = A.mid;
A_midpoint_modif = A_midpoint - diag(diag(A_midpoint)) + diag(diag(A_dol));

delta(:,:,k) = diag(possymmat(A_midpoint_modif));
if sum(diag(delta(:,:,k))==0)
   delta(:,:,k) = delta_E;
end

isMmatrix = 0; nIter = 0; % initialization of iterative computation
maxIter = 100; % maximum number of iterations
while isMmatrix<=0 && nIter<maxIter
    nIter = nIter+1;
    
% Step 3: Check whether B=C*A + 2*C*delta_k is an H-matrix:
% ----- preconditioning matrix C -----
C = inv(A_midpoint_modif + delta(:,:,k));
    
% ----- the interval matrix [B]
B = C*A + 2*C*delta(:,:,k); 

% ----- comparison matrix for [B]------
LH = B.inf; PH =B.sup;
Babssup = sup(abs(B));
Babsinf = inf(abs(B));
B_minabs = min(Babsinf,Babssup);
B_maxabs = max(Babsinf,Babssup);
Bcomp = zeros(n);  % comparison matrix for [B]
for i=1:n
    for j=1:n
        if i~=j
            Bcomp(i,j) = -B_maxabs(i,j);
        elseif LH(i,j)<=0 && PH(i,j)>=0  % 0 is in [a_ij,a-ij]
               Bcomp(i,j) = 0;
        else %LH>0 && PH<0
              Bcomp(i,j) = B_minabs(i,j);
        end
    end
end

% ----- check whether <B> (B_comp) is M-matrix ------
f = ones(n,1);
C = -Bcomp;
d = -1e-6*ones(n,1);
[x,fval,isMmatrix] = linprog(f,C,d,[],[],[],[],[],optimset('display','off'));
% ----- if isMmatrix>0 then <B> is M-matrix ------
 if isMmatrix>0
    alfa_i = diag(delta(:,:,k));
 else
     alfa_i = diag(delta_E);
 end

% Step 4: Construct a new guess for delta_{k+1}
k = k+1;
% delta_{k+1} must maximum separation distance between d_E_max and
% d_{k-1}_max
d_k_max = 1/4*(xU - xL)'*delta(:,:,k-1)*(xU - xL);
factor=1/4;
% The new maximum separation distance will be in the middle between
% d_k_max and d_E_max
d_k1_max = d_k_max + factor*(d_E_max - d_k_max);

if d_k_max<=d_k1_max<=d_E_max
   delta(:,:,k) = diag(d_k1_max);
end

end
alfa_i = min(alfa_i,diag(delta_E));
end